!*******************************************************
!*
!* MODULE for some example initialisation routines
!*
!*******************************************************

MODULE init

  !* Module loaded

  USE constants

  USE algebra

contains

  !****************************************************
  !* 
  !* Routine for the initialization of the matrix
  !*
  !* INPUT/OUTPUT
  !*
  !* a :: matrix to initialize
  !*
  !****************************************************

  SUBROUTINE init_Matrix(a)

    IMPLICIT NONE

    !* Variables declaration

    REAL(idp), DIMENSION(:,:) :: a
    INTEGER                 :: i,n

    !* Code

    n = size(a,1)

    DO i=1,n-1
       a(i,i) = -3.5d0
       a(i,i+1) = 2.d0
       a(i+1,i) = 1.d0
    END DO
    a(n,n) = -3.5d0

  END SUBROUTINE init_Matrix

  !****************************************************
  !* 
  !* Routine for the initialization of the right hand
  !* side
  !*
  !* INPUT 
  !*
  !* a :: the matrix
  !*
  !* INPUT/OUTPUT
  !*
  !* b :: right hand side initialized
  !*
  !****************************************************

  SUBROUTINE init_RHS(a,b)

    IMPLICIT NONE

    !* Variables declaration

    REAL(idp), DIMENSION(:,:)       :: a
    REAL(idp), DIMENSION(:)         :: b
    REAL(idp), DIMENSION(size(b,1)) :: temp

    !* Code

    call init_Solution(temp)

    call MATVEC(a,temp,b)

  END SUBROUTINE init_RHS

  !****************************************************
  !* 
  !* Routine for the initialization of the solution
  !*
  !* INPUT/OUTPUT
  !*
  !* x :: the solution
  !*
  !****************************************************

  SUBROUTINE init_Solution(x)

    IMPLICIT NONE

    !* Variables declaration

    REAL(idp), DIMENSION(:) :: x
    INTEGER                 :: i

    !* Code

    Do i=1,size(x,1)
       x(i) = REAL(i-1)/REAL(size(x,1)-1)
    END Do

  END SUBROUTINE init_Solution

  !****************************************************
  !* 
  !* Routine for the comparison of the solution found
  !* by the user and the real solution
  !*
  !* INPUT
  !*
  !* x :: the solution
  !*
  !****************************************************

  SUBROUTINE compare_To_Solution(x)

    IMPLICIT NONE

    !* Variables declaration

    REAL(idp), DIMENSION(:) :: x
    REAL(idp), DIMENSION(size(x,1)) :: sol

    !* Code

    call init_Solution(sol)

    write(*,*) 'Value of || sol - your_sol || : ', sqrt(DOTPRODUCT(x-sol,x-sol))
  END SUBROUTINE compare_To_Solution

END MODULE init

    
    
